import Sequelize, { Transaction } from 'sequelize';
import { Sequelize as SequelizeTS } from 'sequelize-typescript';
declare class DatabaseService {
    private static LHD;
    /**
     * 상태를 초기화 한다.
     * @deprecated 2022년 11월 25일 현재 더 이상 사용 하지 않는다.
     */
    static init(): void;
    /**
     * id 로 테이블에서 데이터를 가져 온다.
     * return 값이 null 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * null, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param id
     * @param attributes
     * @param transaction
     */
    static getDataOneById<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, id: number | undefined, attributes?: Array<string>, transaction?: Transaction): Promise<T | null | string>;
    /**
     * id(srl) 목록으로 테이블에서 데이터를 가져 온다.
     * return 값이 empty array 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * empty array, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param ids
     * @param attributes
     * @param transaction
     * @param order
     */
    static getDatasByIds<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, ids: Array<number>, attributes?: Array<string>, transaction?: Transaction, order?: Sequelize.Order): Promise<Array<T> | string>;
    /**
     * getDatasByIds 와 같은 역할을 하지만, 데이터가 아니라 데이터의 카운트를 구한다.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param ids
     * @param transaction
     */
    static countDatasByIds<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, ids: Array<number>, transaction?: Transaction): Promise<number | string>;
    /**
     * rid 로 테이블에서 데이터를 가져 온다.
     * return 값이 null 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * null, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param rid
     * @param attributes
     * @param transaction
     */
    static getDataOneByRid<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, rid: string | undefined, attributes?: Array<string>, transaction?: Transaction): Promise<T | null | string>;
    /**
     * rid 목록으로 테이블에서 데이터를 가져 온다.
     * return 값이 empty array 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * empty array, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param rids
     * @param attributes
     * @param transaction
     * @param order
     */
    static getDatasByRids<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, rids: Array<string>, attributes?: Array<string>, transaction?: Transaction, order?: Sequelize.Order): Promise<Array<T> | string>;
    /**
     * getDatasByRids 와 같은 역할을 하지만 데이터가 아니라, 데이터의 카운트를 구한다.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param rids
     * @param transaction
     */
    static countDatasByRids<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, rids: Array<string>, transaction?: Transaction): Promise<number | string>;
    /**
     * 필터 조건으로 테이블에서 데이터를 가져 온다.
     * return 값이 null 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * null, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param filter
     * @param attributes
     * @param transaction
     * @param order
     */
    static getDataOneByFilter<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, filter: {
        [key: string]: any;
    } | undefined, attributes?: Array<string>, transaction?: Transaction, order?: Sequelize.Order): Promise<T | null | string>;
    /**
     * 필터 조건으로 테이블에서 데이터를 가져 온다.
     * return 값이 null 이면 데이터를 발견 하지 못한 것이고, string 이면 에러임(에러 코드 리턴)
     * null, string(DB 에러) 이 아니면 데이터를 발견해서 데이터를 리턴 한 것임.
     * filter, limit 둘 중 하나는 반듯이 존재 해야 한다.
     *
     * order 는 다음과 같이 사용 한다.
     * - [['name', 'DESC']] : name 칼럼으로 desc 정렬
     * - [['name', 'DESC'], ['id', 'ASC']] : name 칼럼으로 desc 정렬하고, 두번째로 id 로 ASC 정렬
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param filter
     * @param attributes
     * @param transaction
     * @param order
     * @param offset
     * @param limit
     */
    static getDatasByFilter<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, filter: {
        [key: string]: any;
    } | undefined, attributes?: Array<string>, transaction?: Transaction, order?: Sequelize.Order, offset?: number, limit?: number): Promise<Array<T> | string>;
    /**
     * getDatasByFilter 와 동일한 역할을 하지만, 데이터가 아니라 데이터의 카운트를 구한다.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param filter
     * @param transaction
     */
    static countDatasByFilter<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, filter: {
        [key: string]: any;
    } | undefined, transaction?: Transaction): Promise<number | string>;
    /**
     * key array 로 데이터 목록을 구한다. 가령 file_id array 로 파일 데이터를 map 형태로 구 할 때 사용 한다.
     * @param reqIpAndLhdAndHttpInfo 로그 헤더
     * @param keys select 조건 key array
     * @param VCreator select 할 테이블
     * @param colName 테이블에서 select 할 칼럼명
     * @param mapKey map 으로 만들때 키가되는 칼럼명(k_칼럼값 형태로 키가 생성된다.)
     * @param cols 테이블에서 특정 칼럼만 가져오고 싶을때 지정 칼럼 목록
     * @param toCond 검색할 테이블 select 시에 추가할 조건
     * @param transaction transaction object
     */
    static getDataMapByKeyArray<T, V>(reqIpAndLhdAndHttpInfo: string, keys: Array<T>, VCreator: {
        new (): V;
    }, colName: string, mapKey: string, cols?: Array<string>, toCond?: {
        [key: string]: any;
    }, transaction?: Transaction): Promise<{
        [key: string]: any;
    }>;
    /**
     * list_order 순서를 변경 한다.
     * raw query 는 함수 내에서 추가 조건이 들어가고, 이에 따라 replacement 도 추가 조건이 들어 간다.
     * list_order 변경 관련 값은 함수 내부에서 처리 되고, 파라미터로 주는 raw query 는 데이터 검색 조건만 들어 가도록 해야 한다.
     * 움직이는 위치에 따라 list_order + 1, list_order -1 을 해야 하기 때문에 파라미터로 넘겨주는 rawQuery 내에 <list_order> 스트링으로
     * 값을 설정 한다.
     *
     * 순수하게 list_order 만 변경 하고, list_order 변경 가능한지 데이터 비교 하지 않는다는 것이 changeListOrderV2 와 다른점 이다.
     *
     * 사용 예제는 한국문화교육의 ContentCategoryAdmRouter.ts 의 카테고리 위치 변경을 참고 하자.
     * @param reqIpAndLhdAndHttpInfo 로그 헤더
     * @param rdb sequelize object
     * @param TCreator 테이블
     * @param rawQuery 부분 raw 쿼리
     * @param replacements raw 쿼리에서 replace 값
     * @param srcId 순서를 변경할 src 의 id
     * @param srcListOrder 순서를 변경할 src 의 list_order
     * @param destListOrder 순서를 변경할 dest 의 list_order
     * @param transaction transaction
     * @throws CkError
     */
    static changeListOrder<T>(reqIpAndLhdAndHttpInfo: string, rdb: SequelizeTS, TCreator: {
        new (): T;
    }, rawQuery: string, replacements: {
        [key: string]: any;
    } | undefined, srcId: number, srcListOrder: number, destListOrder: number, transaction: Transaction): Promise<number>;
    /**
     * list_order 순서를 변경 한다. router 에서 순서 변경하는 것을 함수 한번에 처리 할 수 있도록 묶어 둔 것이다.
     * 간단한 순서 변경에 사용 하고, 추가 제한 조건이 있으면 본 함수를 사용 하지 않도록 한다.
     * 에러 나면 string 을 리턴 하는데, 이런 경우는 호출 하는 곳에서 transaction rollback 처리 해 주어야 한다.
     *
     * changeListOrder 와 다른점은 함수 내부에서 순서를 변경 하려는 데이터를 찾아, 데이터 존재 여부 및 list_order 가 동일한지 비교 한다.
     * 만일 해당 작업을 외부에서 실행 한다면, changeListOrder 을 사용 하도록 한다.
     *
     * 사용 예제는 한국문화교육의 VRCAdminRouter.ts 의 키워드 위치 변경을 참고 하자.
     * @param reqIpAndLhdAndHttpInfo 로그 헤더
     * @param rdb sequelize object
     * @param TCreator 테이블
     * @param srcRid 순서를 변경할 src 의 rid
     * @param destRid 순서를 변경할 dest 의 rid
     * @param rawQueryPart 부분 raw 쿼리
     * @param replacements raw 쿼리에서 replace 값
     * @param transaction transaction
     */
    static changeListOrderV2<T>(reqIpAndLhdAndHttpInfo: string, rdb: SequelizeTS, TCreator: {
        new (): T;
    }, srcRid: string, destRid: string, rawQueryPart: string, replacements: {
        [key: string]: any;
    } | undefined, transaction: Transaction): Promise<number | string>;
    /**
     * 새로운 데이터를 생성 하고, list_order 를 생성된 srl 값과 동일하게 설정 한다.
     * list_order 가 있는 대부분의 데이터 생성시 사용 하면 된다.
     *
     * string 이 리턴 되면, 에러 처리 해야 한다. 에러가 발생 되면 호출 하는 곳에서 transaction rollback 처리 해야 한다.
     * 한국 문화 교육의 VRCKeyword 생성을 참고 한다.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreator
     * @param newData
     * @param transaction
     */
    static createDataWithListOrder<T>(reqIpAndLhdAndHttpInfo: string, TCreator: {
        new (): T;
    }, newData: T, transaction: Transaction): Promise<T | string>;
    /**
     * 데이터를 저장 한다. 정상으로 실행 되면 저장된 데이터를 리턴 하고, 에러이면 에러 코드(string)을 리턴 한다.
     * @param reqIpAndLhdAndHttpInfo
     * @param newData
     * @param transaction
     */
    static createData<T>(reqIpAndLhdAndHttpInfo: string, newData: T, transaction?: Transaction): Promise<T | string>;
    /**
     * 데이터를 업데이트 한다. number 를 리턴하면, 업데이트 데이터 카운트 이며, 스트링이면 에러 코드 이다.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreate
     * @param modData
     * @param where
     * @param transaction
     */
    static updateData<T>(reqIpAndLhdAndHttpInfo: string, TCreate: {
        new (): T;
    }, modData: any, where: any, transaction?: Transaction): Promise<number | string>;
    /**
     * 데이터를 삭제 한다. number 를 리턴하면, 삭제된 데이터 카운트 이며, 스트링 이면 에러 코드 이다.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreate
     * @param where
     * @param transaction
     */
    static deleteData<T>(reqIpAndLhdAndHttpInfo: string, TCreate: {
        new (): T;
    }, where: any, transaction?: Transaction): Promise<number | string>;
    /**
     * 칼럼 값을 증가 시키거나, 감소 시킨다. true 를 리턴하면 처리 성공이고, 스트링 이면 에러 코드 이다.
     * @param reqIpAndLhdAndHttpInfo
     * @param TCreate
     * @param where 쿼리 검색 조건
     * @param cols increase, decrease 시킬 칼럼명. 일반적으로 칼럼 하나만 사용 될 것이지만, 여러개 칼럼을 처리 할 수 있다.
     * @param by increase, decrease 시키는 양. 1 이면 1씩 증가하고, -1 이면 1씩 감소 한다.
     * @param transaction
     */
    static incrementData<T>(reqIpAndLhdAndHttpInfo: string, TCreate: {
        new (): T;
    }, where: any, cols: Array<string>, by: number, transaction?: Transaction): Promise<void | string>;
    /**
     * raw query 를 사용하여 데이터를 select 한다.
     * @param reqIpAndLhdAndHttpInfo
     * @param rawQuery
     * @param replacements
     * @param rdb
     * @param transaction
     */
    static getDataByRawQuery(reqIpAndLhdAndHttpInfo: string, rawQuery: string, rdb: SequelizeTS | number, replacements?: {
        [col: string]: any;
    }, transaction?: Transaction): Promise<Array<{
        [col_name: string]: any;
    }> | string>;
    /**
     * table select 시에 칼럼명을 지정해서 select 할때, 필수로 존재해야 하는 칼럼이 있을 경우, 칼럼명을 추가 한다.
     * undefined 로 select 하면 모든 칼럼을 select 하기 때문에, 원본값(attributes) 가 undefined 이면 undefined 가 리턴 된다.
     * @param columnNames
     * @param attributes
     */
    static addMandatoryColumns(columnNames: Array<string>, attributes?: Array<string>): Array<string> | undefined;
}
export default DatabaseService;
